package mockmaker;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.AntClassLoader;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;

import java.io.File;
import java.util.List;
import java.util.LinkedList;
import java.util.Iterator;

public class AntTask extends Task {

	private MockDoclet mockDoclet = new MockDoclet();
	private List fileSets = new LinkedList();
	private String srcdir, destdir;
	private boolean output = true;
	private Path classpath;

	public void execute() throws BuildException {

		if (srcdir == null && fileSets.size() == 0) {
			throw new BuildException("Specify either srcdir attribute or nested <fileset>");
		}
		if (destdir == null) {
			throw new BuildException("Specify destdir attribute");
		}
		try {
			for (Iterator iterator = fileSets.iterator(); iterator.hasNext();) {
				FileSet fileSet = (FileSet)iterator.next();
				DirectoryScanner ds = fileSet.getDirectoryScanner(getProject());
				String[] files = ds.getIncludedFiles();
				for (int i = 0; i < files.length; i++) {
					String file = files[i];
					if (file.endsWith(".java")) {
						mockDoclet.addFile(new File(ds.getBasedir(),file));
					}
				}
			}
			if (srcdir != null) {
				mockDoclet.setSourceDir(new File(srcdir));
			}
			mockDoclet.setDestDir(new File(destdir));

			if (output) {
				mockDoclet.setLog(System.out);
			}

			if (classpath != null) {
				mockDoclet.addClassLoader(new AntClassLoader(getProject(), classpath));
			}
			mockDoclet.generateMocks();
		}
		catch (Exception e) {
			e.printStackTrace();
			throw new BuildException("Could not generate mock objects", e);
		}
	}

	public void addFileset(FileSet set) {
		fileSets.add(set);
	}

	public void setSrcdir(String srcdir) {
		this.srcdir = srcdir;
	}

	public void setDestdir(String destdir) {
		this.destdir = destdir;
	}

	public void setOutput(boolean output) {
		this.output = output;
	}

	// config options

	public void setAddExpectedValuesFormat(String value) {
		mockDoclet.setConfigProperty("addExpectedValuesFormat", value);
	}

	public void setKeepUsingLastReturnValue(String value) {
		mockDoclet.setConfigProperty("keepUsingLastReturnValue", value);
	}

	public void setSetActualReturnValueFormat(String value) {
		mockDoclet.setConfigProperty("setActualReturnValueFormat", value);
	}

	public void setSetExpectedCallsFormat(String value) {
		mockDoclet.setConfigProperty("setExpectedCallsFormat", value);
	}

	public void setSetActualReturnExceptionFormat(String value) {
		mockDoclet.setConfigProperty("setActualReturnExceptionFormat", value);
	}

	public void setPackageNameFormat(String value) {
		mockDoclet.setConfigProperty("packageNameFormat", value);
	}

	public void setClassNameFormat(String value) {
		mockDoclet.setConfigProperty("classNameFormat", value);
	}

	public void setClasspath(Path classpath) {
		if (this.classpath == null) {
			this.classpath = classpath;
		}
		else {
			this.classpath.append(classpath);
		}
	}

	public Path createClasspath() {
		if (classpath == null) {
			classpath = new Path(getProject());
		}
		return classpath.createPath();
	}

	public void setClasspathRef(Reference r) {
		createClasspath().setRefid(r);
	}
}
