package mockmaker;

import java.io.*;
import java.util.*;

public class RealConfiguration implements Configuration
{
    public static final String DEFAULT_CONFIGURATION_FILENAME = "mockmaker.cfg";

    private Properties myProperties;

    public RealConfiguration( String configFileName )
    {
        super();
        myProperties = new Properties();
        InputStream configPropertyFile = ClassLoader.getSystemResourceAsStream( configFileName );
        if ( configPropertyFile != null )
        {
            try
            {
                myProperties.load( configPropertyFile );
            }
            catch ( IOException ex )
            {
                throw new RuntimeException( "Could not read property file: " + ex );
            }
        }
    }

    public String addExpectedValuesFormat()
    {
        return stringProperty( "addExpectedValuesFormat", "addExpected{0}Values" );
    }

    private boolean isTrue( String value )
    {
        return value != null && value.toLowerCase().equals( "true" );
    }

    private boolean isTrueProperty( String propertyName )
    {
        return isTrue( myProperties.getProperty( propertyName ) );
    }

    public boolean keepUsingLastReturnValue()
    {
        return isTrueProperty( "keepUsingLastReturnValue" );
    }

    public String setActualReturnValueFormat()
    {
        return stringProperty( "setActualReturnValueFormat", "setup{0}" );
    }

    public String setExpectedCallsFormat()
    {
        return stringProperty( "setExpectedCallsFormat", "setExpected{0}Calls" );
    }

    private String stringProperty( String propertyName, String defaultValue )
    {
        String result = myProperties.getProperty( propertyName );
        if ( result == null )
        {
            return defaultValue;
        }
        return result;
    }

    public String setActualReturnExceptionFormat()
    {
        return stringProperty( "setActualReturnExceptionFormat", "setupException{0}" );
    }

    public String packageNameFormat()
    {
        return stringProperty( "packageNameFormat", "{0}" );
    }

    public String classNameFormat()
    {
        return stringProperty( "classNameFormat", "Mock{0}" );
    }

    public void set(String propertyName, String newValue)
    {
        myProperties.setProperty(propertyName, newValue);
    }
}