package mockmaker.structure;

import mockmaker.structure.ClassStructure;
import mockmaker.structure.MethodStructure;
import com.thoughtworks.qdox.model.JavaClass;
import com.thoughtworks.qdox.model.JavaMethod;
import com.thoughtworks.qdox.model.Type;
import com.thoughtworks.qdox.JavaDocBuilder;

import java.util.List;
import java.util.LinkedList;

public class SourceClassStructure implements ClassStructure {

	private JavaClass cls;
	private JavaDocBuilder builder;

	public SourceClassStructure(JavaClass cls, JavaDocBuilder builder) {
		this.cls = cls;
		this.builder = builder;
	}

	public String getName() {
		return cls.getFullyQualifiedName();
	}

	public boolean isInterface() {
		return cls.isInterface();
	}

	public MethodStructure[] getMethods() {
		return getMethodStructures(false);
	}

	public MethodStructure[] getConstructors() {
		return getMethodStructures(true);
	}

	private MethodStructure[] getMethodStructures(boolean constructors) {
		List result = new LinkedList();
		findMethods(cls, constructors, result);
		MethodStructure[] array = new MethodStructure[result.size()];
		result.toArray(array);
		return array;
	}

	private void findMethods(JavaClass currentCls, boolean constructors, List result) {
		// this sucks... refactor me
		JavaMethod[] methods = currentCls.getMethods();
		for(int i = 0; i < methods.length; i++) {
			JavaMethod m = methods[i];
			if (!m.isStatic() && constructors == m.isConstructor()) {
				SourceMethodStructure newMeth = new SourceMethodStructure(m);
				if (!result.contains(newMeth)) {
					result.add(newMeth);
				}
			}
		}
		Type superClassType = currentCls.getSuperClass();
		if (superClassType != null) {
			JavaClass superClass = builder.getClassByName(superClassType.getValue());
			if (superClass != null) {
				findMethods(superClass, constructors, result);
			}
		}
		Type[] implementz = currentCls.getImplements();
		for (int i = 0; i < implementz.length; i++) {
			JavaClass imp = builder.getClassByName(implementz[i].getValue());
			if (imp != null) {
				findMethods(imp, constructors, result);
			}
		}
	}

	// not implemented

	public boolean isArray() {
		return false;
	}

	public int getArrayDimensions() {
		return 0;
	}

	public boolean isPrimitive() {
		return false;
	}

	public String getExpectationName(String name) {
		return null;
	}
}
