package mockmaker.structure;

import com.thoughtworks.qdox.model.Type;
import com.thoughtworks.qdox.model.JavaMethod;

import java.util.Set;
import java.util.HashSet;

public class SourceTypeStructure implements ClassStructure {

	private Type type;
	private JavaMethod mth;
	private String paramName;

	private static Set primitives = new HashSet();

	static {
		primitives.add("void");
		primitives.add("boolean");
		primitives.add("char");
		primitives.add("byte");
		primitives.add("short");
		primitives.add("int");
		primitives.add("long");
		primitives.add("float");
		primitives.add("double");
	}

	SourceTypeStructure(Type type, JavaMethod mth, String paramName) {
		this.type = type;
		this.mth = mth;
		this.paramName = paramName;
	}

	public String getName() {
		return type.getValue();
	}

	public boolean isArray() {
		return type.isArray();
	}

	public int getArrayDimensions() {
		return type.getDimensions();
	}

	public boolean isPrimitive() {
		return primitives.contains(type.getValue());
	}

	public String getExpectationName(String name) {
		return " : " + getName() + " " + paramName;
	}

	// not implemented

	public boolean isInterface() {
		return false;
	}

	public MethodStructure[] getMethods() {
		return new MethodStructure[0];
	}

	public MethodStructure[] getConstructors() {
		return new MethodStructure[0];
	}
}
