package mockmaker.structure.tests;

import junit.framework.TestCase;
import mockmaker.structure.MethodStructure;
import mockmaker.structure.ReflectionConstructorStructure;

public class ReflectionConstructorStructureTest extends TestCase {

	private static class Dummy {
		public Dummy() {}
		public Dummy(String a) throws IllegalArgumentException {}
		public Dummy(String a, int b) throws IllegalArgumentException, java.io.IOException {}
	}

	public ReflectionConstructorStructureTest(String s) {
		super(s);
	}

	public void testNoParams() throws Exception {
		MethodStructure m = new ReflectionConstructorStructure(Dummy.class.getConstructor(new Class[] {}));
		assertEquals(0, m.getParameterTypes().length);
	}

	public void testOneParam() throws Exception {
		MethodStructure m = new ReflectionConstructorStructure(Dummy.class.getConstructor(new Class[] {String.class}));
		assertEquals(1, m.getParameterTypes().length);
		assertEquals("java.lang.String", m.getParameterTypes()[0].getName());
	}

	public void testTwoParams() throws Exception {
		MethodStructure m = new ReflectionConstructorStructure(Dummy.class.getConstructor(new Class[] {String.class, Integer.TYPE}));
		assertEquals(2, m.getParameterTypes().length);
		assertEquals("java.lang.String", m.getParameterTypes()[0].getName());
		assertEquals("int", m.getParameterTypes()[1].getName());
	}

	public void testNoExceptions() throws Exception {
		MethodStructure m = new ReflectionConstructorStructure(Dummy.class.getConstructor(new Class[] {}));
		assertEquals(0, m.getExceptions().length);
	}

	public void testOneException() throws Exception {
		MethodStructure m = new ReflectionConstructorStructure(Dummy.class.getConstructor(new Class[] {String.class}));
		assertEquals(1, m.getExceptions().length);
		assertEquals("java.lang.IllegalArgumentException", m.getExceptions()[0].getName());
	}

	public void testTwoExceptions() throws Exception {
		MethodStructure m = new ReflectionConstructorStructure(Dummy.class.getConstructor(new Class[] {String.class, Integer.TYPE}));
		assertEquals(2, m.getExceptions().length);
		assertEquals("java.lang.IllegalArgumentException", m.getExceptions()[0].getName());
		assertEquals("java.io.IOException", m.getExceptions()[1].getName());
	}

}
