package mockmaker.structure.tests;

import junit.framework.TestCase;
import mockmaker.structure.MethodStructure;
import mockmaker.structure.ReflectionMethodStructure;

public class ReflectionMethodStructureTest extends TestCase {

	public ReflectionMethodStructureTest(String s) {
		super(s);
	}

	static interface Dummy {
		void threeParams(Object a, int b, String c);
		void oneException() throws IllegalArgumentException;
		void twoExceptions() throws IllegalArgumentException, java.io.IOException;
		String toString();
		String notToString();
		String toString(String s);
		String toString(StringBuffer s);
	}

  public void testName() throws Exception {
		MethodStructure m = new ReflectionMethodStructure(Object.class.getMethod("toString", null));
		assertEquals("toString", m.getName());
	}

	public void testReturnType() throws Exception {
		MethodStructure m = new ReflectionMethodStructure(Object.class.getMethod("toString", null));
		assertEquals("java.lang.String", m.getReturnType().getName());
	}

	public void testNoParameterTypes() throws Exception {
		MethodStructure m = new ReflectionMethodStructure(Object.class.getMethod("toString", null));
		assertEquals(0, m.getParameterTypes().length);
	}

	public void testOneParameterType() throws Exception {
		MethodStructure m = new ReflectionMethodStructure(Object.class.getMethod("equals", new Class[]{Object.class}));
		assertEquals(1, m.getParameterTypes().length);
		assertEquals("java.lang.Object", m.getParameterTypes()[0].getName());
	}

	public void testThreeParameterTypes() throws Exception {
		MethodStructure m = new ReflectionMethodStructure(Dummy.class.getMethod("threeParams", new Class[]{Object.class, Integer.TYPE, String.class}));
		assertEquals(3, m.getParameterTypes().length);
		assertEquals("java.lang.Object", m.getParameterTypes()[0].getName());
		assertEquals("int", m.getParameterTypes()[1].getName());
		assertEquals("java.lang.String", m.getParameterTypes()[2].getName());
	}

	public void testNoExceptions() throws Exception {
		MethodStructure m = new ReflectionMethodStructure(Object.class.getMethod("toString", null));
		assertEquals(0, m.getExceptions().length);
	}

	public void testOneException() throws Exception {
		MethodStructure m = new ReflectionMethodStructure(Dummy.class.getMethod("oneException", null));
		assertEquals(1, m.getExceptions().length);
		assertEquals("java.lang.IllegalArgumentException", m.getExceptions()[0].getName());
	}

	public void testTwoExceptions() throws Exception {
		MethodStructure m = new ReflectionMethodStructure(Dummy.class.getMethod("twoExceptions", null));
		assertEquals(2, m.getExceptions().length);
		assertEquals("java.lang.IllegalArgumentException", m.getExceptions()[0].getName());
		assertEquals("java.io.IOException", m.getExceptions()[1].getName());
	}

	public void testEquals() throws Exception {
		MethodStructure m1 = new ReflectionMethodStructure(Object.class.getMethod("toString", null));
		MethodStructure m2 = new ReflectionMethodStructure(Dummy.class.getMethod("toString", null));
		MethodStructure m3 = new ReflectionMethodStructure(Dummy.class.getMethod("toString", new Class[] {String.class}));
		MethodStructure m4 = new ReflectionMethodStructure(Dummy.class.getMethod("notToString", null));
		MethodStructure m5 = new ReflectionMethodStructure(Dummy.class.getMethod("toString", new Class[] {StringBuffer.class}));
		assertEquals(m1, m2);
		assertEquals(m2, m1);
		assertTrue(!m2.equals(m3));
		assertTrue(!m2.equals(m4));
		assertTrue(!m3.equals(m5));
	}


}
