package mockmaker.structure.tests;

import junit.framework.TestCase;
import mockmaker.structure.ClassStructure;
import mockmaker.structure.SourceClassStructure;
import mockmaker.structure.MethodStructure;
import com.thoughtworks.qdox.JavaDocBuilder;

import java.io.StringReader;

public class SourceClassStructureTest extends TestCase {

	private ClassStructure thing, theThing, staticClass, returnStuff,
		exceptionStuff, paramStuff, constructors, advancedThing, advancedTheThing;

	public SourceClassStructureTest(String s) {
		super(s);
	}

	protected void setUp() throws Exception {
		super.setUp();
		String f1 = ""
			+ "package test;"
			+ "public interface Thing {"
			+ "  int doStuff(long x);"
			+ "  void nothing();"
			+ "}";
		String f2 = ""
			+ "package test.impl;"
			+ "import test.*;"
			+ "public class TheThing implements Thing {"
			+ "  public int doStuff(long x) { return 0; }"
			+ "  public void nothing() { }"
			+ "}";
		String f3 = ""
			+ "package test.stat;"
			+ "public class StaticClass {"
			+ "  public void method() { }"
			+ "  public static void staticMethod() { }"
			+ "}";
		String f4 = ""
			+ "package test.returntypes;"
			+ "public class ReturnStuff {"
			+ "  public void nothing() {}"
			+ "  public int primitive() {}"
			+ "  public boolean bool() {}"
			+ "  public Object obj() {}"
			+ "  public String str() {}"
			+ "  public Thing thing() {}"
			+ "  public long[] longArray() {}"
			+ "  public Object[] objectArray() {}"
			+ "  public Thing[] thingArray() {}"
			+ "  public long[][][] tripleArray() {}"
			+ "}";
		String f5 = ""
			+ "package test.exceptions;"
			+ "import java.io.IOException;"
			+ "class CustomException extends Exception {}"
			+ "public interface ExceptionStuff {"
			+ "  void none();"
			+ "  void one() throws IOException;"
			+ "  void two() throws IOException, CustomException;"
			+ "}";
		String f6= ""
			+ "package test.params;"
			+ "public interface ParamStuff {"
			+ "  void none();"
			+ "  void one(int x);"
			+ "  void three(int x, boolean z, Object o);"
			+ "  void array(int[] x);"
			+ "  void multiArray(String[][] x);"
			+ "}";
		String f7= ""
			+ "package test;"
			+ "public class Constructors {"
			+ "  Constructors() {}"
			+ "  Constructors(String a) {}"
			+ "  void method() {}"
			+ "}";
		String f8 = ""
			+ "package blah;"
			+ "public class AdvancedTheThing extends test.impl.TheThing {"
			+ "  public int advanced() {}"
			+ "}";
		String f9 = ""
			+ "package blah;"
			+ "public interface AdvancedThing extends test.Thing {"
			+ "  int advanced();"
			+ "}";
		JavaDocBuilder builder = new JavaDocBuilder();
		builder.addSource(new StringReader(f1));
		builder.addSource(new StringReader(f2));
		builder.addSource(new StringReader(f3));
		builder.addSource(new StringReader(f4));
		builder.addSource(new StringReader(f5));
		builder.addSource(new StringReader(f6));
		builder.addSource(new StringReader(f7));
		builder.addSource(new StringReader(f8));
		builder.addSource(new StringReader(f9));
		thing = new SourceClassStructure(builder.getClassByName("test.Thing"), builder);
		theThing = new SourceClassStructure(builder.getClassByName("test.impl.TheThing"), builder);
		staticClass = new SourceClassStructure(builder.getClassByName("test.stat.StaticClass"), builder);
		returnStuff = new SourceClassStructure(builder.getClassByName("test.returntypes.ReturnStuff"), builder);
		exceptionStuff = new SourceClassStructure(builder.getClassByName("test.exceptions.ExceptionStuff"), builder);
		paramStuff = new SourceClassStructure(builder.getClassByName("test.params.ParamStuff"), builder);
		constructors = new SourceClassStructure(builder.getClassByName("test.Constructors"), builder);
		advancedThing = new SourceClassStructure(builder.getClassByName("blah.AdvancedThing"), builder);
		advancedTheThing = new SourceClassStructure(builder.getClassByName("blah.AdvancedTheThing"), builder);
	}

	public void testName() throws Exception {
		assertEquals("test.Thing", thing.getName());
		assertEquals("test.impl.TheThing", theThing.getName());
	}

	public void testIsInterface() throws Exception {
		assertTrue(thing.isInterface());
		assertTrue(!theThing.isInterface());
	}

	public void testMethodCount() throws Exception {
		assertEquals(2, thing.getMethods().length);
		assertEquals(2, theThing.getMethods().length);
	}

	public void testMethodNames() throws Exception {
		assertEquals("doStuff", thing.getMethods()[0].getName());
		assertEquals("nothing", thing.getMethods()[1].getName());
		assertEquals("doStuff", theThing.getMethods()[0].getName());
		assertEquals("nothing", theThing.getMethods()[1].getName());
	}

	public void testMethodReturnTypes() throws Exception {
		assertEquals("int", thing.getMethods()[0].getReturnType().getName());
		assertEquals("void", thing.getMethods()[1].getReturnType().getName());
	}

	public void testNoStaticMethods() throws Exception {
		assertEquals(1, staticClass.getMethods().length);
		assertEquals("method", staticClass.getMethods()[0].getName());
	}

	public void testPrimitivesMethods() throws Exception {
		// true
		assertTrue(returnStuff.getMethods()[0].getReturnType().isPrimitive());
		assertTrue(returnStuff.getMethods()[1].getReturnType().isPrimitive());
		assertTrue(returnStuff.getMethods()[2].getReturnType().isPrimitive());
		assertTrue(returnStuff.getMethods()[6].getReturnType().isPrimitive());
		assertTrue(returnStuff.getMethods()[9].getReturnType().isPrimitive());

		// false
		assertTrue(!returnStuff.getMethods()[3].getReturnType().isPrimitive());
		assertTrue(!returnStuff.getMethods()[4].getReturnType().isPrimitive());
		assertTrue(!returnStuff.getMethods()[5].getReturnType().isPrimitive());
	}

	public void testArrayMethods() throws Exception {
		//true
		assertTrue(returnStuff.getMethods()[6].getReturnType().isArray());
		assertTrue(returnStuff.getMethods()[7].getReturnType().isArray());
		assertTrue(returnStuff.getMethods()[8].getReturnType().isArray());
		assertTrue(returnStuff.getMethods()[9].getReturnType().isArray());

		// false
		assertTrue(!returnStuff.getMethods()[0].getReturnType().isArray());
		assertTrue(!returnStuff.getMethods()[1].getReturnType().isArray());
		assertTrue(!returnStuff.getMethods()[2].getReturnType().isArray());
		assertTrue(!returnStuff.getMethods()[3].getReturnType().isArray());
		assertTrue(!returnStuff.getMethods()[4].getReturnType().isArray());
		assertTrue(!returnStuff.getMethods()[5].getReturnType().isArray());
	}

	public void testArrayDimensions() throws Exception {
		assertEquals(0,returnStuff.getMethods()[0].getReturnType().getArrayDimensions());
		assertEquals(0,returnStuff.getMethods()[1].getReturnType().getArrayDimensions());
		assertEquals(0,returnStuff.getMethods()[2].getReturnType().getArrayDimensions());
		assertEquals(0,returnStuff.getMethods()[3].getReturnType().getArrayDimensions());
		assertEquals(0,returnStuff.getMethods()[4].getReturnType().getArrayDimensions());
		assertEquals(0,returnStuff.getMethods()[5].getReturnType().getArrayDimensions());
		assertEquals(1,returnStuff.getMethods()[6].getReturnType().getArrayDimensions());
		assertEquals(1,returnStuff.getMethods()[7].getReturnType().getArrayDimensions());
		assertEquals(3,returnStuff.getMethods()[9].getReturnType().getArrayDimensions());
	}

	public void testExceptionCount() throws Exception {
		assertEquals(0, exceptionStuff.getMethods()[0].getExceptions().length);
		assertEquals(1, exceptionStuff.getMethods()[1].getExceptions().length);
		assertEquals(2, exceptionStuff.getMethods()[2].getExceptions().length);
	}

	public void testExceptions() throws Exception {
		ClassStructure[] exceptions = exceptionStuff.getMethods()[2].getExceptions();
		assertEquals("java.io.IOException", exceptions[0].getName());
		assertEquals("test.exceptions.CustomException", exceptions[1].getName());
	}

	public void testParameters() throws Exception {
		// void none();
		MethodStructure m1 = paramStuff.getMethods()[0];
		assertEquals(0, m1.getParameterTypes().length);

		// void one(int x);
		MethodStructure m2 = paramStuff.getMethods()[1];
		assertEquals(1, m2.getParameterTypes().length);
		assertEquals("int", m2.getParameterTypes()[0].getName());
		assertEquals(0, m2.getParameterTypes()[0].getArrayDimensions());

		// void three(int x, boolean z, Object o);;
		MethodStructure m3 = paramStuff.getMethods()[2];
		assertEquals(3, m3.getParameterTypes().length);
		assertEquals("int", m3.getParameterTypes()[0].getName());
		assertEquals("boolean", m3.getParameterTypes()[1].getName());
		assertEquals("java.lang.Object", m3.getParameterTypes()[2].getName());

		// void array(int[] x);
		MethodStructure m4 = paramStuff.getMethods()[3];
		assertEquals(1, m4.getParameterTypes().length);
		assertEquals("int", m4.getParameterTypes()[0].getName());
		assertEquals(1, m4.getParameterTypes()[0].getArrayDimensions());

		// void multiArray(String[][] x);
		MethodStructure m5 = paramStuff.getMethods()[4];
		assertEquals(1, m5.getParameterTypes().length);
		assertEquals("java.lang.String", m5.getParameterTypes()[0].getName());
		assertEquals(2, m5.getParameterTypes()[0].getArrayDimensions());

	}

	public void testConstructors() throws Exception {
		assertEquals(2, constructors.getConstructors().length);
		assertEquals(1, constructors.getMethods().length);
		assertEquals(0, constructors.getConstructors()[0].getParameterTypes().length);
		assertEquals(1, constructors.getConstructors()[1].getParameterTypes().length);
	}

	public void testClassInheritance() throws Exception {
		assertEquals("advanced", advancedTheThing.getMethods()[0].getName());
		assertEquals("doStuff", advancedTheThing.getMethods()[1].getName());
		assertEquals("nothing", advancedTheThing.getMethods()[2].getName());
	}

}
