package mockmaker.tests;

import java.io.*;
import junit.framework.*;
import junit.textui.TestRunner;
import mockmaker.*;

public class AcceptanceTests extends TestCase {
	private StringWriter myStringWriter;
	private PrintWriter myPrintWriter;

	private CodeWriter myCodeWriter;

	private StringWriter myExpectingStringWriter;
	private PrintWriter myExpectingPrintWriter;
public AcceptanceTests(String name) {
	super(name);
}
public static void main(String args[]) {
	TestRunner.main(new String[]{"mockmaker.tests.AcceptanceTests"});
}
public void setUp() throws IOException {
	myStringWriter = new StringWriter();
	myPrintWriter = new PrintWriter(myStringWriter);

	myCodeWriter = new RealCodeWriter(myPrintWriter);

	myExpectingStringWriter = new StringWriter();
	myExpectingPrintWriter = new PrintWriter(myExpectingStringWriter);
	myConfiguration = new RealConfiguration("nonExistantFileGivesDefaultValues");
}
public static Test suite ( ) {
	TestSuite suite= new TestSuite(AcceptanceTests.class);
	return suite;
}
public void testAcceptableInterface() throws Exception {
	MockMaker aMockMaker = new MockMaker("mockmaker.tests.AcceptableInterface", myCodeWriter, myConfiguration);
	aMockMaker.make();
	myExpectingPrintWriter.println("import mockmaker.ReturnValues;");
	myExpectingPrintWriter.println("import mockmaker.VoidReturnValues;");
	myExpectingPrintWriter.println("import mockmaker.ExceptionalReturnValue;");
	myExpectingPrintWriter.println("import com.mockobjects.*;");
	myExpectingPrintWriter.println("import mockmaker.tests.AcceptableInterface;");
	myExpectingPrintWriter.println("import mockmaker.tests.OneVoidMethod;");
	myExpectingPrintWriter.println("import java.io.IOException;");

	myExpectingPrintWriter.println("\npublic class MockAcceptableInterface implements AcceptableInterface{");
	myExpectingPrintWriter.println("	private ExpectationCounter myDifficultMethodCalls = new ExpectationCounter(\"MockAcceptableInterface.difficultMethod(String, OneVoidMethod[])\");");
	myExpectingPrintWriter.println("	private ReturnValues myActualDifficultMethodReturnValues = new ReturnValues(\"MockAcceptableInterface.difficultMethod(String, OneVoidMethod[])\", false);");
	myExpectingPrintWriter.println("	private ExpectationList myDifficultMethodParameter0Values = new ExpectationList(\"MockAcceptableInterface.difficultMethod(String, OneVoidMethod[]) arg0 values\");");
	myExpectingPrintWriter.println("	private ExpectationList myDifficultMethodParameter1Values = new ExpectationList(\"MockAcceptableInterface.difficultMethod(String, OneVoidMethod[]) arg1 values\");");
	myExpectingPrintWriter.println("	private ExpectationCounter myMinimalMethodCalls = new ExpectationCounter(\"MockAcceptableInterface.minimalMethod()\");");
    myExpectingPrintWriter.println("	private ReturnValues myActualMinimalMethodReturnValues = new VoidReturnValues(\"MockAcceptableInterface.minimalMethod()\", false);");
	myExpectingPrintWriter.println("	private ExpectationCounter myNormalMethodCalls = new ExpectationCounter(\"MockAcceptableInterface.normalMethod(Integer)\");");
	myExpectingPrintWriter.println("	private ReturnValues myActualNormalMethodReturnValues = new ReturnValues(\"MockAcceptableInterface.normalMethod(Integer)\", false);");
	myExpectingPrintWriter.println("	private ExpectationList myNormalMethodParameter0Values = new ExpectationList(\"MockAcceptableInterface.normalMethod(Integer) arg0 values\");");

	myExpectingPrintWriter.println("\n	public void setExpectedDifficultMethodCalls(int calls){");
	myExpectingPrintWriter.println("		myDifficultMethodCalls.setExpected(calls);");
	myExpectingPrintWriter.println("	}");

	myExpectingPrintWriter.println("\n	public void addExpectedDifficultMethodValues(String arg0, OneVoidMethod[] arg1){");
	myExpectingPrintWriter.println("		myDifficultMethodParameter0Values.addExpected(arg0);");
	myExpectingPrintWriter.println("		myDifficultMethodParameter1Values.addExpected(arg1);");
	myExpectingPrintWriter.println("	}");

	myExpectingPrintWriter.println("\n	public int[] difficultMethod(String arg0, OneVoidMethod[] arg1){");
	myExpectingPrintWriter.println("		myDifficultMethodCalls.inc();");
	myExpectingPrintWriter.println("		myDifficultMethodParameter0Values.addActual(arg0);");
	myExpectingPrintWriter.println("		myDifficultMethodParameter1Values.addActual(arg1);");
	myExpectingPrintWriter.println("		Object nextReturnValue = myActualDifficultMethodReturnValues.getNext();");
	myExpectingPrintWriter.println("		if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)");
	myExpectingPrintWriter.println("			throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();");
	myExpectingPrintWriter.println("		return (int[]) nextReturnValue;");
	myExpectingPrintWriter.println("	}");

	myExpectingPrintWriter.println("\n	public void setupExceptionDifficultMethod(Throwable arg){");
	myExpectingPrintWriter.println("		myActualDifficultMethodReturnValues.add(new ExceptionalReturnValue(arg));");
	myExpectingPrintWriter.println("	}");

	myExpectingPrintWriter.println("\n	public void setupDifficultMethod(int[] arg){");
	myExpectingPrintWriter.println("		myActualDifficultMethodReturnValues.add(arg);");
	myExpectingPrintWriter.println("	}");

	myExpectingPrintWriter.println("\n	public void setExpectedMinimalMethodCalls(int calls){");
	myExpectingPrintWriter.println("		myMinimalMethodCalls.setExpected(calls);");
	myExpectingPrintWriter.println("	}");

	myExpectingPrintWriter.println("\n	public void minimalMethod() throws IOException{");
	myExpectingPrintWriter.println("		myMinimalMethodCalls.inc();");
    myExpectingPrintWriter.println("		Object nextReturnValue = myActualMinimalMethodReturnValues.getNext();");
    myExpectingPrintWriter.println("		if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof IOException)");
    myExpectingPrintWriter.println("			throw (IOException)((ExceptionalReturnValue)nextReturnValue).getException();");
	myExpectingPrintWriter.println("		if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)");
    myExpectingPrintWriter.println("			throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();");
	myExpectingPrintWriter.println("	}");

    myExpectingPrintWriter.println("\n	public void setupExceptionMinimalMethod(Throwable arg){");
	myExpectingPrintWriter.println("		myActualMinimalMethodReturnValues.add(new ExceptionalReturnValue(arg));");
	myExpectingPrintWriter.println("	}");

	myExpectingPrintWriter.println("\n	public void setExpectedNormalMethodCalls(int calls){");
	myExpectingPrintWriter.println("		myNormalMethodCalls.setExpected(calls);");
	myExpectingPrintWriter.println("	}");

	myExpectingPrintWriter.println("\n	public void addExpectedNormalMethodValues(Integer arg0){");
	myExpectingPrintWriter.println("		myNormalMethodParameter0Values.addExpected(arg0);");
	myExpectingPrintWriter.println("	}");

	myExpectingPrintWriter.println("\n	public String normalMethod(Integer arg0){");
	myExpectingPrintWriter.println("		myNormalMethodCalls.inc();");
	myExpectingPrintWriter.println("		myNormalMethodParameter0Values.addActual(arg0);");
	myExpectingPrintWriter.println("		Object nextReturnValue = myActualNormalMethodReturnValues.getNext();");
    myExpectingPrintWriter.println("		if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)");
	myExpectingPrintWriter.println("			throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();");
	myExpectingPrintWriter.println("		return (String) nextReturnValue;");
	myExpectingPrintWriter.println("	}");

	myExpectingPrintWriter.println("\n	public void setupExceptionNormalMethod(Throwable arg){");
	myExpectingPrintWriter.println("		myActualNormalMethodReturnValues.add(new ExceptionalReturnValue(arg));");
	myExpectingPrintWriter.println("	}");

	myExpectingPrintWriter.println("\n	public void setupNormalMethod(String arg){");
	myExpectingPrintWriter.println("		myActualNormalMethodReturnValues.add(arg);");
	myExpectingPrintWriter.println("	}");

	myExpectingPrintWriter.println("\n	public void verify(){");
	myExpectingPrintWriter.println("		myDifficultMethodCalls.verify();");
	myExpectingPrintWriter.println("		myDifficultMethodParameter0Values.verify();");
	myExpectingPrintWriter.println("		myDifficultMethodParameter1Values.verify();");
	myExpectingPrintWriter.println("		myMinimalMethodCalls.verify();");
	myExpectingPrintWriter.println("		myNormalMethodCalls.verify();");
	myExpectingPrintWriter.println("		myNormalMethodParameter0Values.verify();");
	myExpectingPrintWriter.println("	}");
	myExpectingPrintWriter.println("}");
	myExpectingPrintWriter.flush();
	assertEquals(myExpectingStringWriter.toString().length(), myStringWriter.toString().length());
}
public void testInterfaceWithNoMethods() throws Exception {
	MockMaker aMockMaker = new MockMaker("mockmaker.tests.InterfaceWithNoMethods", myCodeWriter, myConfiguration);
	aMockMaker.make();

	myExpectingPrintWriter.println("import mockmaker.ReturnValues;");
	myExpectingPrintWriter.println("import mockmaker.VoidReturnValues;");
	myExpectingPrintWriter.println("import mockmaker.ExceptionalReturnValue;");
	myExpectingPrintWriter.println("import com.mockobjects.*;");
	myExpectingPrintWriter.println("import mockmaker.tests.InterfaceWithNoMethods;");

	myExpectingPrintWriter.println("\npublic class MockInterfaceWithNoMethods implements InterfaceWithNoMethods{");

	myExpectingPrintWriter.println("\n	public void verify(){");
	myExpectingPrintWriter.println("	}");
	myExpectingPrintWriter.println("}");
	myExpectingPrintWriter.flush();

	assertEquals(myExpectingStringWriter.toString(),myStringWriter.toString());
}

	private Configuration myConfiguration;
}