package mockmaker.tests;

import java.io.*;
import junit.framework.*;
import junit.textui.TestRunner;
import mockmaker.*;

public class CodeWriterTest extends TestCase {
	private StringWriter myStringWriter;
	private PrintWriter myPrintWriter;

	private CodeWriter myCodeWriter;

	private StringWriter myExpectingStringWriter;
	private PrintWriter myExpectingPrintWriter;
public CodeWriterTest(String name) {
	super(name);
}
public static void main(String args[]) {
	TestRunner.main(new String[]{"mockmaker.tests.CodeWriterTest"});
}
public void setUp(){
	myStringWriter = new StringWriter();
	myPrintWriter = new PrintWriter(myStringWriter);

	myCodeWriter = new RealCodeWriter(myPrintWriter);

	myExpectingStringWriter = new StringWriter();
	myExpectingPrintWriter = new PrintWriter(myExpectingStringWriter);
}
public static Test suite ( ) {
	TestSuite suite= new TestSuite(CodeWriterTest.class);
	return suite;
}
public void testFinish() throws Exception {
	myCodeWriter.finish();
	myPrintWriter.flush();

	myExpectingPrintWriter.println("}");
	myExpectingPrintWriter.flush();

	assertEquals(myExpectingStringWriter.toString(),myStringWriter.toString());
}
public void testWritingClassDeclaration() throws Exception {
	myCodeWriter.writeClassDeclaration("MockTestTester","TestTester");
	myPrintWriter.flush();

	myExpectingPrintWriter.println("\npublic class MockTestTester implements TestTester{");
	myExpectingPrintWriter.flush();

	assertEquals(myExpectingStringWriter.toString(),myStringWriter.toString());
}
public void testWritingImport() throws Exception {
	myCodeWriter.writeImport("uk.co.jesterinfo.jester.*");
	myPrintWriter.flush();

	myExpectingPrintWriter.println("import uk.co.jesterinfo.jester.*;");
	myExpectingPrintWriter.flush();

	assertEquals(myExpectingStringWriter.toString(),myStringWriter.toString());
}
public void testWritingPackage() throws Exception {
  myCodeWriter.writePackage("uk.co.jesterinfo.jester");
  myPrintWriter.flush();

  myExpectingPrintWriter.println("package uk.co.jesterinfo.jester;");
  myExpectingPrintWriter.flush();

  assertEquals(myExpectingStringWriter.toString(),myStringWriter.toString());
}
public void testWritingInstanceVariableDeclaration() throws Exception {
	myCodeWriter.writeInstanceVariableDeclaration("Foo","aFooBar");
	myPrintWriter.flush();

	myExpectingPrintWriter.println("	private Foo aFooBar;");
	myExpectingPrintWriter.flush();

	assertEquals(myExpectingStringWriter.toString(),myStringWriter.toString());
}
public void testWritingInstanceVariableDeclarationWithInitialValue() throws Exception {
	myCodeWriter.writeInstanceVariableDeclaration("Foo","aFooBar", "new RealFoo()");
	myPrintWriter.flush();

	myExpectingPrintWriter.println("	private Foo aFooBar = new RealFoo();");
	myExpectingPrintWriter.flush();

	assertEquals(myExpectingStringWriter.toString(),myStringWriter.toString());
}
public void testWritingMethodDeclaration() throws Exception {
	myCodeWriter.writeMethodDeclaration("int",
										"parseInt",
										new String[]{"String aString","boolean aCondition"},
										new String[]{"aString.foo();","return aCondition ? 1 : 2;"});
	myPrintWriter.flush();

	myExpectingPrintWriter.println("\n	public int parseInt(String aString, boolean aCondition){");
	myExpectingPrintWriter.println("		aString.foo();");
	myExpectingPrintWriter.println("		return aCondition ? 1 : 2;");
	myExpectingPrintWriter.println("	}");
	myExpectingPrintWriter.flush();

	assertEquals(myExpectingStringWriter.toString(),myStringWriter.toString());
}

public void testWritingMethodDeclarationThrowsExceptions() throws Exception {
	myCodeWriter.writeMethodDeclarationThrowsExceptions("int",
										"parseInt",
										new String[]{"String aString","boolean aCondition"},
										new String[]{"AnException","AnotherException"},
										new String[]{"aString.foo();","return aCondition ? 1 : 2;"});
	myPrintWriter.flush();

	myExpectingPrintWriter.println("\n	public int parseInt(String aString, boolean aCondition) throws AnException, AnotherException{");
	myExpectingPrintWriter.println("		aString.foo();");
	myExpectingPrintWriter.println("		return aCondition ? 1 : 2;");
	myExpectingPrintWriter.println("	}");
	myExpectingPrintWriter.flush();

	assertEquals(myExpectingStringWriter.toString(),myStringWriter.toString());
}

public void testWritingConstructor() throws Exception {
	myCodeWriter.writeConstructorMethodDeclaration("MockTestTester",new String[]{"TestTester"});
	myPrintWriter.flush();

	myExpectingPrintWriter.println("public MockTestTester(TestTester arg0){");
	myExpectingPrintWriter.println("	super(arg0);");
	myExpectingPrintWriter.println("}");
	myExpectingPrintWriter.flush();

	assertEquals(myExpectingStringWriter.toString(),myStringWriter.toString());
}

public void testWritingSubclassDeclaration() throws Exception {
	myCodeWriter.writeSubclassDeclaration("MockTestTester","TestTester");
	myPrintWriter.flush();

	myExpectingPrintWriter.println("\npublic class MockTestTester extends TestTester{");
	myExpectingPrintWriter.flush();

	assertEquals(myExpectingStringWriter.toString(),myStringWriter.toString());
}
}