package mockmaker.tests;

import junit.framework.*;
import junit.textui.TestRunner;
import mockmaker.*;
import mockmaker.structure.SourceClassStructure;
import mockmaker.structure.ClassStructure;

import java.text.*;
import java.io.StringReader;

import com.thoughtworks.qdox.JavaDocBuilder;
import com.thoughtworks.qdox.model.JavaClass;

public class MockMakerTest extends TestCase {
	private MockCodeWriter myMockCodeWriter;
public MockMakerTest(String name) {
	super(name);
}
public static void main(String args[]) {
	TestRunner.main(new String[]{"mockmaker.tests.MockMakerTest"});
}
private MockMaker newMockMaker(String interfaceName) {
	return new MockMaker(interfaceName, myMockCodeWriter, new TestConfiguration());
}
public void setUp() {
	myMockCodeWriter = new MockCodeWriter();
}
public static Test suite ( ) {
	TestSuite suite= new TestSuite(MockMakerTest.class);
	return suite;
}
public void testMakeMockForClass() throws Exception {
	myMockCodeWriter.setExpectedFinishCalls(1);
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.VoidReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ExceptionalReturnValue");
	myMockCodeWriter.addExpectedWriteImportValues("com.mockobjects.*");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.ClassNotInterface");

	myMockCodeWriter.addExpectedWriteSubclassDeclarationValues("XMockClassNotInterface","ClassNotInterface");
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualNewMethodReturnValues","new VoidReturnValues(\"XMockClassNotInterface.newMethod()\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myNewMethodCalls","new ExpectationCounter(\"XMockClassNotInterface.newMethod()\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("NewMethod"),
																new String[]{"Throwable arg"},
																new String[]{"myActualNewMethodReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("NewMethod"),
																new String[]{"int calls"},
																new String[]{"myNewMethodCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"newMethod",
																new String[]{"myNewMethodCalls.inc();",
                                                                             "Object nextReturnValue = myActualNewMethodReturnValues.getNext();",
                                                                             "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                             "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"
                                                                });
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	"verify",
																new String[]{"myNewMethodCalls.verify();"});
	myMockCodeWriter.addExpectedWriteConstructorMethodDeclarationValues("XMockClassNotInterface",new String[]{"String"});

	MockMaker aMockMaker = newMockMaker("mockmaker.tests.ClassNotInterface");
	aMockMaker.make();

	verify();
}
public void testMakeMockForInterfaceWithInheritedMethod() throws Exception {
	myMockCodeWriter.setExpectedFinishCalls(1);
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.VoidReturnValues");
    myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ExceptionalReturnValue");
	myMockCodeWriter.addExpectedWriteImportValues("com.mockobjects.*");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.AnotherVoidMethod");

	myMockCodeWriter.addExpectedWriteClassDeclarationValues("XMockAnotherVoidMethod","AnotherVoidMethod");
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualChildMethodReturnValues","new VoidReturnValues(\"XMockAnotherVoidMethod.childMethod()\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myChildMethodCalls","new ExpectationCounter(\"XMockAnotherVoidMethod.childMethod()\")");
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualNewMethodReturnValues","new VoidReturnValues(\"XMockAnotherVoidMethod.newMethod()\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myNewMethodCalls","new ExpectationCounter(\"XMockAnotherVoidMethod.newMethod()\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("ChildMethod"),
																new String[]{"Throwable arg"},
																new String[]{"myActualChildMethodReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("ChildMethod"),
																new String[]{"int calls"},
																new String[]{"myChildMethodCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"childMethod",
																new String[]{"myChildMethodCalls.inc();",
                                                                             "Object nextReturnValue = myActualChildMethodReturnValues.getNext();",
                                                                             "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                             "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"
                                                                });
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("NewMethod"),
																new String[]{"Throwable arg"},
																new String[]{"myActualNewMethodReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("NewMethod"),
																new String[]{"int calls"},
																new String[]{"myNewMethodCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"newMethod",
																new String[]{"myNewMethodCalls.inc();",
                                                                             "Object nextReturnValue = myActualNewMethodReturnValues.getNext();",
                                                                             "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                             "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"
                                                                });
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	"verify",
																new String[]{
																	"myChildMethodCalls.verify();",
																	"myNewMethodCalls.verify();"});

	MockMaker aMockMaker = newMockMaker("mockmaker.tests.AnotherVoidMethod");
	aMockMaker.make();

	verify();
}
public void testMakeMockForInterfaceWithNoMethods() throws Exception {
	myMockCodeWriter.setExpectedFinishCalls(1);
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.VoidReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ExceptionalReturnValue");
	myMockCodeWriter.addExpectedWriteImportValues("com.mockobjects.*");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.InterfaceWithNoMethods");

	myMockCodeWriter.addExpectedWriteClassDeclarationValues("XMockInterfaceWithNoMethods","InterfaceWithNoMethods");
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues("verify",new String[0]);

	MockMaker aMockMaker = newMockMaker("mockmaker.tests.InterfaceWithNoMethods");
	aMockMaker.make();

	verify();
}
public void testMakeMockForInterfaceWithOneArrayParameterMethod() throws Exception {
	myMockCodeWriter.setExpectedFinishCalls(1);
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.VoidReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ExceptionalReturnValue");
	myMockCodeWriter.addExpectedWriteImportValues("com.mockobjects.*");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.OneStringArrayParameterMethod");

	myMockCodeWriter.addExpectedWriteClassDeclarationValues("XMockOneStringArrayParameterMethod","OneStringArrayParameterMethod");
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualNewMethodReturnValues","new VoidReturnValues(\"XMockOneStringArrayParameterMethod.newMethod(String[][])\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myNewMethodCalls","new ExpectationCounter(\"XMockOneStringArrayParameterMethod.newMethod(String[][])\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myNewMethodParameter0Values","new ExpectationList(\"XMockOneStringArrayParameterMethod.newMethod(String[][]) arg0 values\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("NewMethod"),
																new String[]{"Throwable arg"},
																new String[]{"myActualNewMethodReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("NewMethod"),
																new String[]{"int calls"},
																new String[]{"myNewMethodCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	addExpectedValuesString("NewMethod"),
																new String[]{"String[][] arg0"},
																new String[]{"myNewMethodParameter0Values.addExpected(arg0);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"newMethod",
																new String[]{"String[][] arg0"},
																new String[]{
																	"myNewMethodCalls.inc();",
																	"myNewMethodParameter0Values.addActual(arg0);",
                                                                    "Object nextReturnValue = myActualNewMethodReturnValues.getNext();",
                                                                    "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                    "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"
                                                                });
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	"verify",
																new String[]{
																	"myNewMethodCalls.verify();",
																	"myNewMethodParameter0Values.verify();"});

	MockMaker aMockMaker = newMockMaker("mockmaker.tests.OneStringArrayParameterMethod");
	aMockMaker.make();

	verify();
}
public void testMakeMockForInterfaceWithOneStringParameterMethod() throws Exception {
	myMockCodeWriter.setExpectedFinishCalls(1);
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.VoidReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ExceptionalReturnValue");
	myMockCodeWriter.addExpectedWriteImportValues("com.mockobjects.*");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.OneStringParameterMethod");

	myMockCodeWriter.addExpectedWriteClassDeclarationValues("XMockOneStringParameterMethod","OneStringParameterMethod");
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualNewMethodReturnValues","new VoidReturnValues(\"XMockOneStringParameterMethod.newMethod(String)\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myNewMethodCalls","new ExpectationCounter(\"XMockOneStringParameterMethod.newMethod(String)\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myNewMethodParameter0Values","new ExpectationList(\"XMockOneStringParameterMethod.newMethod(String) arg0 values\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("NewMethod"),
																new String[]{"Throwable arg"},
																new String[]{"myActualNewMethodReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("NewMethod"),
																new String[]{"int calls"},
																new String[]{"myNewMethodCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	addExpectedValuesString("NewMethod"),
																new String[]{"String arg0"},
																new String[]{"myNewMethodParameter0Values.addExpected(arg0);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"newMethod",
																new String[]{"String arg0"},
																new String[]{
																	"myNewMethodCalls.inc();",
																	"myNewMethodParameter0Values.addActual(arg0);",
                                                                    "Object nextReturnValue = myActualNewMethodReturnValues.getNext();",
                                                                    "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                    "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"
                                                                });
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	"verify",
																new String[]{
																	"myNewMethodCalls.verify();",
																	"myNewMethodParameter0Values.verify();"
                                                                });

	MockMaker aMockMaker = newMockMaker("mockmaker.tests.OneStringParameterMethod");
	aMockMaker.make();

	verify();
}
public void testMakeMockForInterfaceWithOneStringReturningMethod() throws Exception {
	myMockCodeWriter.setExpectedFinishCalls(1);
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.VoidReturnValues");
  myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ExceptionalReturnValue");
	myMockCodeWriter.addExpectedWriteImportValues("com.mockobjects.*");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.OneStringReturningMethod");

	myMockCodeWriter.addExpectedWriteClassDeclarationValues("XMockOneStringReturningMethod","OneStringReturningMethod");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myToStringCalls","new ExpectationCounter(\"XMockOneStringReturningMethod.toString()\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualToStringReturnValues","new ReturnValues(\"XMockOneStringReturningMethod.toString()\", true)");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("ToString"),
																new String[]{"Throwable arg"},
																new String[]{"myActualToStringReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("ToString"),
																new String[]{"int calls"},
																new String[]{"myToStringCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"String",
																"toString",
																new String[]{	"myToStringCalls.inc();",
																	 			"Object nextReturnValue = myActualToStringReturnValues.getNext();",
                                                                                "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                                "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();",
																				"return (String) nextReturnValue;"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnValueString("ToString"),
																new String[]{"String arg"},
																new String[]{"myActualToStringReturnValues.add(arg);"});
   	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	"verify",
	   															new String[]{"myToStringCalls.verify();"});

	MockMaker aMockMaker = newMockMaker("mockmaker.tests.OneStringReturningMethod");
	aMockMaker.make();

	verify();
}
public void testMakeMockForInterfaceWithOneVoidMethod() throws Exception {
	myMockCodeWriter.setExpectedFinishCalls(1);
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.VoidReturnValues");
  myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ExceptionalReturnValue");
	myMockCodeWriter.addExpectedWriteImportValues("com.mockobjects.*");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.OneVoidMethod");

	myMockCodeWriter.addExpectedWriteClassDeclarationValues("XMockOneVoidMethod","OneVoidMethod");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualNewMethodReturnValues","new VoidReturnValues(\"XMockOneVoidMethod.newMethod()\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myNewMethodCalls","new ExpectationCounter(\"XMockOneVoidMethod.newMethod()\")");
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("NewMethod"),
																new String[]{"Throwable arg"},
																new String[]{"myActualNewMethodReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("NewMethod"),
																new String[]{"int calls"},
																new String[]{"myNewMethodCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"newMethod",
																new String[]{"myNewMethodCalls.inc();",
                                                                        "Object nextReturnValue = myActualNewMethodReturnValues.getNext();",
                                                                        "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                        "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"
                                                                });
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	"verify",
																new String[]{"myNewMethodCalls.verify();"});

	MockMaker aMockMaker = newMockMaker("mockmaker.tests.OneVoidMethod");
	aMockMaker.make();

	verify();
}
public void testMakeMockForNonExistantClass() {
	try{
		myMockCodeWriter.setExpectedFinishCalls(0);
		myMockCodeWriter.setExpectedWriteClassDeclarationCalls(0);
		myMockCodeWriter.setExpectedWriteImportCalls(0);
		myMockCodeWriter.setExpectedWriteInstanceVariableDeclarationCalls(0);
		myMockCodeWriter.setExpectedWriteMethodDeclarationCalls(0);

		newMockMaker("mockmaker.tests.NonExistantClass").make();
		fail("expected exception to be thrown");
	}catch(ClassNotFoundException ex){
		//expected exception to be thrown
	}

	verify();
}
private void verify(){
	myMockCodeWriter.verify();
}

	private static final Configuration TEST_CONFIGURATION = new TestConfiguration();

private String addExpectedValuesString(String methodName) {
	return MessageFormat.format(TEST_CONFIGURATION.addExpectedValuesFormat(), new String[] { methodName });
}

private String setActualReturnValueString(String methodName) {
	return MessageFormat.format(TEST_CONFIGURATION.setActualReturnValueFormat(), new String[] { methodName });
}

private String setActualReturnExceptionString(String methodName) {
	return MessageFormat.format(TEST_CONFIGURATION.setActualReturnExceptionFormat(), new String[] { methodName });
}

private String setExpectedCallsString(String methodName) {
	return MessageFormat.format(TEST_CONFIGURATION.setExpectedCallsFormat(), new String[] { methodName });
}

public void testMakeMockForInterfaceWithExceptionThrowingMethod() throws Exception {
	myMockCodeWriter.setExpectedFinishCalls(1);
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.VoidReturnValues");
  myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ExceptionalReturnValue");
	myMockCodeWriter.addExpectedWriteImportValues("com.mockobjects.*");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.ExceptionThrowingMethod");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.SomeCheckedException");
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualNewMethodReturnValues","new VoidReturnValues(\"XMockExceptionThrowingMethod.newMethod()\", true)");
	myMockCodeWriter.addExpectedWriteClassDeclarationValues("XMockExceptionThrowingMethod","ExceptionThrowingMethod");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myNewMethodCalls","new ExpectationCounter(\"XMockExceptionThrowingMethod.newMethod()\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("NewMethod"),
																new String[]{"Throwable arg"},
																new String[]{"myActualNewMethodReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("NewMethod"),
																new String[]{"int calls"},
																new String[]{"myNewMethodCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"void",
																"newMethod",
																new String[0],
																new String[]{"SomeCheckedException"},
																new String[]{"myNewMethodCalls.inc();",
                                                                    "Object nextReturnValue = myActualNewMethodReturnValues.getNext();",
                                                                    "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof SomeCheckedException)",
                                                                    "	throw (SomeCheckedException)((ExceptionalReturnValue)nextReturnValue).getException();",
                                                                    "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                    "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"
                                                                });
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	"verify",
																new String[]{"myNewMethodCalls.verify();"});

	MockMaker aMockMaker = newMockMaker("mockmaker.tests.ExceptionThrowingMethod");
	aMockMaker.make();

	verify();
}

public void testMakeMockForInterfaceWithOnePrimitiveReturningMethod() throws Exception {
	myMockCodeWriter.setExpectedFinishCalls(1);
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.VoidReturnValues");
  myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ExceptionalReturnValue");
	myMockCodeWriter.addExpectedWriteImportValues("com.mockobjects.*");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.OneIntReturningMethod");
	myMockCodeWriter.addExpectedWriteClassDeclarationValues("XMockOneIntReturningMethod","OneIntReturningMethod");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myToIntCalls","new ExpectationCounter(\"XMockOneIntReturningMethod.toInt()\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualToIntReturnValues","new ReturnValues(\"XMockOneIntReturningMethod.toInt()\", true)");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("ToInt"),
																new String[]{"Throwable arg"},
																new String[]{"myActualToIntReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("ToInt"),
																new String[]{"int calls"},
																new String[]{"myToIntCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"int",
																"toInt",
																new String[]{	"myToIntCalls.inc();",
																	 			"Object nextReturnValue = myActualToIntReturnValues.getNext();",
                                                                                "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                                "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();",
																				"return ((Integer) nextReturnValue).intValue();"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnValueString("ToInt"),
																new String[]{"int arg"},
																new String[]{"myActualToIntReturnValues.add(new Integer(arg));"});
   	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	"verify",
	   															new String[]{"myToIntCalls.verify();"});

	MockMaker aMockMaker = newMockMaker("mockmaker.tests.OneIntReturningMethod");
	aMockMaker.make();

	verify();
}

public void testMakeMockForInterfaceWithOverloadedMethod() throws Exception {
	myMockCodeWriter.setExpectedFinishCalls(1);
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.VoidReturnValues");
  myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ExceptionalReturnValue");
	myMockCodeWriter.addExpectedWriteImportValues("com.mockobjects.*");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.OverloadedMethod");
	myMockCodeWriter.addExpectedWriteClassDeclarationValues("XMockOverloadedMethod","OverloadedMethod");
	//one arg version
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualOverloadedMethodIntReturnValues","new ReturnValues(\"XMockOverloadedMethod.overloadedMethod(int)\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myOverloadedMethodIntCalls","new ExpectationCounter(\"XMockOverloadedMethod.overloadedMethod(int)\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myOverloadedMethodIntParameter0Values","new ExpectationList(\"XMockOverloadedMethod.overloadedMethod(int) arg0 values\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("OverloadedMethodInt"),
																new String[]{"Throwable arg"},
																new String[]{"myActualOverloadedMethodIntReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("OverloadedMethodInt"),
																new String[]{"int calls"},
																new String[]{"myOverloadedMethodIntCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	addExpectedValuesString("OverloadedMethodInt"),
																new String[]{	"int arg0"},
																new String[]{	"myOverloadedMethodIntParameter0Values.addExpected(new Integer(arg0));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnValueString("OverloadedMethodInt"),
																new String[]{"String arg"},
																new String[]{"myActualOverloadedMethodIntReturnValues.add(arg);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"String",
																"overloadedMethod",
																new String[]{"int arg0"},
																new String[]{	"myOverloadedMethodIntCalls.inc();",
																				"myOverloadedMethodIntParameter0Values.addActual(new Integer(arg0));",
																				"Object nextReturnValue = myActualOverloadedMethodIntReturnValues.getNext();",
                                                                                "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                                "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();",
																				"return (String) nextReturnValue;"});
	//two args version
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualOverloadedMethodIntStringReturnValues","new ReturnValues(\"XMockOverloadedMethod.overloadedMethod(int, String)\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myOverloadedMethodIntStringCalls","new ExpectationCounter(\"XMockOverloadedMethod.overloadedMethod(int, String)\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myOverloadedMethodIntStringParameter0Values","new ExpectationList(\"XMockOverloadedMethod.overloadedMethod(int, String) arg0 values\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myOverloadedMethodIntStringParameter1Values","new ExpectationList(\"XMockOverloadedMethod.overloadedMethod(int, String) arg1 values\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("OverloadedMethodIntString"),
																new String[]{"Throwable arg"},
																new String[]{"myActualOverloadedMethodIntStringReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("OverloadedMethodIntString"),
																new String[]{"int calls"},
																new String[]{"myOverloadedMethodIntStringCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	addExpectedValuesString("OverloadedMethodIntString"),
																new String[]{	"int arg0",
																				"String arg1"},
																new String[]{	"myOverloadedMethodIntStringParameter0Values.addExpected(new Integer(arg0));",
																				"myOverloadedMethodIntStringParameter1Values.addExpected(arg1);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnValueString("OverloadedMethodIntString"),
																new String[]{"String arg"},
																new String[]{"myActualOverloadedMethodIntStringReturnValues.add(arg);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"String",
																"overloadedMethod",
																new String[]{"int arg0", "String arg1"},
																new String[]{	"myOverloadedMethodIntStringCalls.inc();",
																				"myOverloadedMethodIntStringParameter0Values.addActual(new Integer(arg0));",
																				"myOverloadedMethodIntStringParameter1Values.addActual(arg1);",
																				"Object nextReturnValue = myActualOverloadedMethodIntStringReturnValues.getNext();",
                                                                                "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                                "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();",
																				"return (String) nextReturnValue;"});
	//both
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	"verify",
																new String[]{
																	"myOverloadedMethodIntStringCalls.verify();",
																	"myOverloadedMethodIntStringParameter0Values.verify();",
																	"myOverloadedMethodIntStringParameter1Values.verify();",
																	"myOverloadedMethodIntCalls.verify();",
																	"myOverloadedMethodIntParameter0Values.verify();"});

	MockMaker aMockMaker = newMockMaker("mockmaker.tests.OverloadedMethod");
	aMockMaker.make();

	verify();
}

public void testMakeMockForInterfaceWithOverloadedMethodWithArrayParameter() throws Exception {
	myMockCodeWriter.setExpectedFinishCalls(1);
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.VoidReturnValues");
  myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ExceptionalReturnValue");
	myMockCodeWriter.addExpectedWriteImportValues("com.mockobjects.*");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.OverloadedMethodWithArray");
	myMockCodeWriter.addExpectedWriteClassDeclarationValues("XMockOverloadedMethodWithArray","OverloadedMethodWithArray");
	//one arg version
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualOverloadedMethodIntArrayArrayReturnValues","new VoidReturnValues(\"XMockOverloadedMethodWithArray.overloadedMethod(int[][])\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myOverloadedMethodIntArrayArrayCalls","new ExpectationCounter(\"XMockOverloadedMethodWithArray.overloadedMethod(int[][])\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myOverloadedMethodIntArrayArrayParameter0Values","new ExpectationList(\"XMockOverloadedMethodWithArray.overloadedMethod(int[][]) arg0 values\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("OverloadedMethodIntArrayArray"),
																new String[]{"Throwable arg"},
																new String[]{"myActualOverloadedMethodIntArrayArrayReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("OverloadedMethodIntArrayArray"),
																new String[]{"int calls"},
																new String[]{"myOverloadedMethodIntArrayArrayCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	addExpectedValuesString("OverloadedMethodIntArrayArray"),
																new String[]{	"int[][] arg0"},
																new String[]{	"myOverloadedMethodIntArrayArrayParameter0Values.addExpected(arg0);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"overloadedMethod",
																new String[]{"int[][] arg0"},
																new String[]{	"myOverloadedMethodIntArrayArrayCalls.inc();",
																				"myOverloadedMethodIntArrayArrayParameter0Values.addActual(arg0);",
                                                                                "Object nextReturnValue = myActualOverloadedMethodIntArrayArrayReturnValues.getNext();",
                                                                                "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                                "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"
                                                                });
	//two args version
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualOverloadedMethodIntArrayArrayStringReturnValues","new VoidReturnValues(\"XMockOverloadedMethodWithArray.overloadedMethod(int[][], String)\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myOverloadedMethodIntArrayArrayStringCalls","new ExpectationCounter(\"XMockOverloadedMethodWithArray.overloadedMethod(int[][], String)\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myOverloadedMethodIntArrayArrayStringParameter0Values","new ExpectationList(\"XMockOverloadedMethodWithArray.overloadedMethod(int[][], String) arg0 values\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myOverloadedMethodIntArrayArrayStringParameter1Values","new ExpectationList(\"XMockOverloadedMethodWithArray.overloadedMethod(int[][], String) arg1 values\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("OverloadedMethodIntArrayArrayString"),
																new String[]{"Throwable arg"},
																new String[]{"myActualOverloadedMethodIntArrayArrayStringReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("OverloadedMethodIntArrayArrayString"),
																new String[]{"int calls"},
																new String[]{"myOverloadedMethodIntArrayArrayStringCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	addExpectedValuesString("OverloadedMethodIntArrayArrayString"),
																new String[]{	"int[][] arg0",
																				"String arg1"},
																new String[]{	"myOverloadedMethodIntArrayArrayStringParameter0Values.addExpected(arg0);",
																				"myOverloadedMethodIntArrayArrayStringParameter1Values.addExpected(arg1);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"overloadedMethod",
																new String[]{"int[][] arg0", "String arg1"},
																new String[]{	"myOverloadedMethodIntArrayArrayStringCalls.inc();",
																				"myOverloadedMethodIntArrayArrayStringParameter0Values.addActual(arg0);",
																				"myOverloadedMethodIntArrayArrayStringParameter1Values.addActual(arg1);",
                                                                                "Object nextReturnValue = myActualOverloadedMethodIntArrayArrayStringReturnValues.getNext();",
                                                                                "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                                "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"
                                                                });
	//both
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	"verify",
																new String[]{
																	"myOverloadedMethodIntArrayArrayStringCalls.verify();",
																	"myOverloadedMethodIntArrayArrayStringParameter0Values.verify();",
																	"myOverloadedMethodIntArrayArrayStringParameter1Values.verify();",
																	"myOverloadedMethodIntArrayArrayCalls.verify();",
																	"myOverloadedMethodIntArrayArrayParameter0Values.verify();"});

	MockMaker aMockMaker = newMockMaker("mockmaker.tests.OverloadedMethodWithArray");
	aMockMaker.make();

	verify();
}

public void testMakeMockForInterfaceWithPrimitiveTypes() throws Exception {
	myMockCodeWriter.setExpectedFinishCalls(1);
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.VoidReturnValues");
  myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ExceptionalReturnValue");
	myMockCodeWriter.addExpectedWriteImportValues("com.mockobjects.*");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.NonObjectParameterMethods");
	myMockCodeWriter.addExpectedWriteClassDeclarationValues("XMockNonObjectParameterMethods","NonObjectParameterMethods");
	//int
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualNewMethodIntReturnValues","new VoidReturnValues(\"XMockNonObjectParameterMethods.newMethod(int)\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myNewMethodIntCalls","new ExpectationCounter(\"XMockNonObjectParameterMethods.newMethod(int)\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myNewMethodIntParameter0Values","new ExpectationList(\"XMockNonObjectParameterMethods.newMethod(int) arg0 values\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("NewMethodInt"),
																new String[]{"Throwable arg"},
																new String[]{"myActualNewMethodIntReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("NewMethodInt"),
																new String[]{"int calls"},
																new String[]{"myNewMethodIntCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	addExpectedValuesString("NewMethodInt"),
																new String[]{	"int arg0"},
																new String[]{	"myNewMethodIntParameter0Values.addExpected(new Integer(arg0));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"newMethod",
																new String[]{"int arg0"},
																new String[]{	"myNewMethodIntCalls.inc();",
																				"myNewMethodIntParameter0Values.addActual(new Integer(arg0));",
                                                                                "Object nextReturnValue = myActualNewMethodIntReturnValues.getNext();",
                                                                                "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                                "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"
                                                                });
	//boolean
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualNewMethodBooleanReturnValues","new VoidReturnValues(\"XMockNonObjectParameterMethods.newMethod(boolean)\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myNewMethodBooleanCalls","new ExpectationCounter(\"XMockNonObjectParameterMethods.newMethod(boolean)\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myNewMethodBooleanParameter0Values","new ExpectationList(\"XMockNonObjectParameterMethods.newMethod(boolean) arg0 values\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("NewMethodBoolean"),
																new String[]{"Throwable arg"},
																new String[]{"myActualNewMethodBooleanReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("NewMethodBoolean"),
																new String[]{"int calls"},
																new String[]{"myNewMethodBooleanCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	addExpectedValuesString("NewMethodBoolean"),
																new String[]{	"boolean arg0"},
																new String[]{	"myNewMethodBooleanParameter0Values.addExpected(new Boolean(arg0));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"newMethod",
																new String[]{"boolean arg0"},
																new String[]{	"myNewMethodBooleanCalls.inc();",
																				"myNewMethodBooleanParameter0Values.addActual(new Boolean(arg0));",
                                                                                "Object nextReturnValue = myActualNewMethodBooleanReturnValues.getNext();",
                                                                                "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                                "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"});
	//short
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualNewMethodShortReturnValues","new VoidReturnValues(\"XMockNonObjectParameterMethods.newMethod(short)\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myNewMethodShortCalls","new ExpectationCounter(\"XMockNonObjectParameterMethods.newMethod(short)\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myNewMethodShortParameter0Values","new ExpectationList(\"XMockNonObjectParameterMethods.newMethod(short) arg0 values\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("NewMethodShort"),
																new String[]{"Throwable arg"},
																new String[]{"myActualNewMethodShortReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("NewMethodShort"),
																new String[]{"int calls"},
																new String[]{"myNewMethodShortCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	addExpectedValuesString("NewMethodShort"),
																new String[]{	"short arg0"},
																new String[]{	"myNewMethodShortParameter0Values.addExpected(new Short(arg0));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"newMethod",
																new String[]{"short arg0"},
																new String[]{	"myNewMethodShortCalls.inc();",
																				"myNewMethodShortParameter0Values.addActual(new Short(arg0));",
                                                                                "Object nextReturnValue = myActualNewMethodShortReturnValues.getNext();",
                                                                                "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                                "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"});
	//byte
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualNewMethodByteReturnValues","new VoidReturnValues(\"XMockNonObjectParameterMethods.newMethod(byte)\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myNewMethodByteCalls","new ExpectationCounter(\"XMockNonObjectParameterMethods.newMethod(byte)\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myNewMethodByteParameter0Values","new ExpectationList(\"XMockNonObjectParameterMethods.newMethod(byte) arg0 values\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("NewMethodByte"),
																new String[]{"Throwable arg"},
																new String[]{"myActualNewMethodByteReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("NewMethodByte"),
																new String[]{"int calls"},
																new String[]{"myNewMethodByteCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	addExpectedValuesString("NewMethodByte"),
																new String[]{	"byte arg0"},
																new String[]{	"myNewMethodByteParameter0Values.addExpected(new Byte(arg0));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"newMethod",
																new String[]{"byte arg0"},
																new String[]{	"myNewMethodByteCalls.inc();",
																				"myNewMethodByteParameter0Values.addActual(new Byte(arg0));",
                                                                                "Object nextReturnValue = myActualNewMethodByteReturnValues.getNext();",
                                                                                "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                                "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"});
	//char
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualNewMethodCharReturnValues","new VoidReturnValues(\"XMockNonObjectParameterMethods.newMethod(char)\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myNewMethodCharCalls","new ExpectationCounter(\"XMockNonObjectParameterMethods.newMethod(char)\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myNewMethodCharParameter0Values","new ExpectationList(\"XMockNonObjectParameterMethods.newMethod(char) arg0 values\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("NewMethodChar"),
																new String[]{"Throwable arg"},
																new String[]{"myActualNewMethodCharReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("NewMethodChar"),
																new String[]{"int calls"},
																new String[]{"myNewMethodCharCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	addExpectedValuesString("NewMethodChar"),
																new String[]{	"char arg0"},
																new String[]{	"myNewMethodCharParameter0Values.addExpected(new Character(arg0));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"newMethod",
																new String[]{"char arg0"},
																new String[]{	"myNewMethodCharCalls.inc();",
																				"myNewMethodCharParameter0Values.addActual(new Character(arg0));",
                                                                                "Object nextReturnValue = myActualNewMethodCharReturnValues.getNext();",
                                                                                "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                                "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"});
	//double
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualNewMethodDoubleReturnValues","new VoidReturnValues(\"XMockNonObjectParameterMethods.newMethod(double)\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myNewMethodDoubleCalls","new ExpectationCounter(\"XMockNonObjectParameterMethods.newMethod(double)\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myNewMethodDoubleParameter0Values","new ExpectationList(\"XMockNonObjectParameterMethods.newMethod(double) arg0 values\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("NewMethodDouble"),
																new String[]{"Throwable arg"},
																new String[]{"myActualNewMethodDoubleReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("NewMethodDouble"),
																new String[]{"int calls"},
																new String[]{"myNewMethodDoubleCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	addExpectedValuesString("NewMethodDouble"),
																new String[]{	"double arg0"},
																new String[]{	"myNewMethodDoubleParameter0Values.addExpected(new Double(arg0));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"newMethod",
																new String[]{"double arg0"},
																new String[]{	"myNewMethodDoubleCalls.inc();",
																				"myNewMethodDoubleParameter0Values.addActual(new Double(arg0));",
                                                                                "Object nextReturnValue = myActualNewMethodDoubleReturnValues.getNext();",
                                                                                "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                                "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"});
	//float
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualNewMethodFloatReturnValues","new VoidReturnValues(\"XMockNonObjectParameterMethods.newMethod(float)\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myNewMethodFloatCalls","new ExpectationCounter(\"XMockNonObjectParameterMethods.newMethod(float)\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myNewMethodFloatParameter0Values","new ExpectationList(\"XMockNonObjectParameterMethods.newMethod(float) arg0 values\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("NewMethodFloat"),
																new String[]{"Throwable arg"},
																new String[]{"myActualNewMethodFloatReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("NewMethodFloat"),
																new String[]{"int calls"},
																new String[]{"myNewMethodFloatCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	addExpectedValuesString("NewMethodFloat"),
																new String[]{	"float arg0"},
																new String[]{	"myNewMethodFloatParameter0Values.addExpected(new Float(arg0));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"newMethod",
																new String[]{"float arg0"},
																new String[]{	"myNewMethodFloatCalls.inc();",
																				"myNewMethodFloatParameter0Values.addActual(new Float(arg0));",
                                                                                "Object nextReturnValue = myActualNewMethodFloatReturnValues.getNext();",
                                                                                "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                                "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"});
	//long
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualNewMethodLongReturnValues","new VoidReturnValues(\"XMockNonObjectParameterMethods.newMethod(long)\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myNewMethodLongCalls","new ExpectationCounter(\"XMockNonObjectParameterMethods.newMethod(long)\")");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myNewMethodLongParameter0Values","new ExpectationList(\"XMockNonObjectParameterMethods.newMethod(long) arg0 values\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("NewMethodLong"),
																new String[]{"Throwable arg"},
																new String[]{"myActualNewMethodLongReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("NewMethodLong"),
																new String[]{"int calls"},
																new String[]{"myNewMethodLongCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	addExpectedValuesString("NewMethodLong"),
																new String[]{	"long arg0"},
																new String[]{	"myNewMethodLongParameter0Values.addExpected(new Long(arg0));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"newMethod",
																new String[]{"long arg0"},
																new String[]{	"myNewMethodLongCalls.inc();",
																				"myNewMethodLongParameter0Values.addActual(new Long(arg0));",
                                                                                "Object nextReturnValue = myActualNewMethodLongReturnValues.getNext();",
                                                                                "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                                "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"});
	//
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	"verify",
																new String[]{
																	"myNewMethodLongCalls.verify();",
																	"myNewMethodLongParameter0Values.verify();",
																	"myNewMethodFloatCalls.verify();",
																	"myNewMethodFloatParameter0Values.verify();",
																	"myNewMethodDoubleCalls.verify();",
																	"myNewMethodDoubleParameter0Values.verify();",
																	"myNewMethodByteCalls.verify();",
																	"myNewMethodByteParameter0Values.verify();",
																	"myNewMethodCharCalls.verify();",
																	"myNewMethodCharParameter0Values.verify();",
																	"myNewMethodBooleanCalls.verify();",
																	"myNewMethodBooleanParameter0Values.verify();",
																	"myNewMethodShortCalls.verify();",
																	"myNewMethodShortParameter0Values.verify();",
																	"myNewMethodIntCalls.verify();",
																	"myNewMethodIntParameter0Values.verify();"});

	MockMaker aMockMaker = newMockMaker("mockmaker.tests.NonObjectParameterMethods");
	aMockMaker.make();

	verify();
}

public void testMakeMockForSubclass() throws Exception {
	myMockCodeWriter.setExpectedFinishCalls(1);
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ReturnValues");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.VoidReturnValues");
  myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ExceptionalReturnValue");
	myMockCodeWriter.addExpectedWriteImportValues("com.mockobjects.*");
	myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.Subclass");
	myMockCodeWriter.addExpectedWriteSubclassDeclarationValues("XMockSubclass","Subclass");
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualSubclassMethodReturnValues","new VoidReturnValues(\"XMockSubclass.subclassMethod()\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","mySubclassMethodCalls","new ExpectationCounter(\"XMockSubclass.subclassMethod()\")");
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualNewMethodReturnValues","new VoidReturnValues(\"XMockSubclass.newMethod()\", true)");
	myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myNewMethodCalls","new ExpectationCounter(\"XMockSubclass.newMethod()\")");
    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("SubclassMethod"),
																new String[]{"Throwable arg"},
																new String[]{"myActualSubclassMethodReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("SubclassMethod"),
																new String[]{"int calls"},
																new String[]{"mySubclassMethodCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"subclassMethod",
																new String[]{"mySubclassMethodCalls.inc();",
                                                                    "Object nextReturnValue = myActualSubclassMethodReturnValues.getNext();",
                                                                    "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                    "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"
                                                                });

    myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setActualReturnExceptionString("NewMethod"),
																new String[]{"Throwable arg"},
																new String[]{"myActualNewMethodReturnValues.add(new ExceptionalReturnValue(arg));"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	setExpectedCallsString("NewMethod"),
																new String[]{"int calls"},
																new String[]{"myNewMethodCalls.setExpected(calls);"});
	myMockCodeWriter.addExpectedWriteMethodDeclarationThrowsExceptionsValues(	"newMethod",
																new String[]{"myNewMethodCalls.inc();",
                                                                "Object nextReturnValue = myActualNewMethodReturnValues.getNext();",
                                                                "if (nextReturnValue instanceof ExceptionalReturnValue && ((ExceptionalReturnValue)nextReturnValue).getException() instanceof RuntimeException)",
                                                                "	throw (RuntimeException)((ExceptionalReturnValue)nextReturnValue).getException();"
                                                                });

	myMockCodeWriter.addExpectedWriteMethodDeclarationValues(	"verify",
																new String[]{	"mySubclassMethodCalls.verify();",
																				"myNewMethodCalls.verify();"});
	myMockCodeWriter.addExpectedWriteConstructorMethodDeclarationValues("XMockSubclass",new String[]{"String"});

	MockMaker aMockMaker = newMockMaker("mockmaker.tests.Subclass");
	aMockMaker.make();

	verify();
}

	public void testMakeMockFromSource() throws Exception {
		myMockCodeWriter.setExpectedFinishCalls(1);
		myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ReturnValues");
		myMockCodeWriter.addExpectedWriteImportValues("mockmaker.VoidReturnValues");
		myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ExceptionalReturnValue");
		myMockCodeWriter.addExpectedWriteImportValues("com.mockobjects.*");
		myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.InterfaceWithNoMethods");

		myMockCodeWriter.addExpectedWriteClassDeclarationValues("XMockInterfaceWithNoMethods","InterfaceWithNoMethods");
		myMockCodeWriter.addExpectedWriteMethodDeclarationValues("verify",new String[0]);

		ClassStructure classStructure = createSourceClassStructure(
			"package mockmaker.tests; public interface InterfaceWithNoMethods {}");
		MockMaker aMockMaker = new MockMaker(classStructure, myMockCodeWriter, new TestConfiguration());
		aMockMaker.make();

		verify();
	}

	public void testMakeMockWithPackage() throws Exception {
		myMockCodeWriter.setExpectedFinishCalls(1);
		myMockCodeWriter.addExpectedWritePackageValues("Xmockmaker.tests.mytestpackage");
		myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ReturnValues");
		myMockCodeWriter.addExpectedWriteImportValues("mockmaker.VoidReturnValues");
		myMockCodeWriter.addExpectedWriteImportValues("mockmaker.ExceptionalReturnValue");
		myMockCodeWriter.addExpectedWriteImportValues("com.mockobjects.*");
		myMockCodeWriter.addExpectedWriteImportValues("mockmaker.tests.InterfaceWithNoMethods");

		myMockCodeWriter.addExpectedWriteClassDeclarationValues("XMockInterfaceWithNoMethods","InterfaceWithNoMethods");
		myMockCodeWriter.addExpectedWriteMethodDeclarationValues("verify",new String[0]);

		ClassStructure classStructure = createSourceClassStructure(
			"package mockmaker.tests; public interface InterfaceWithNoMethods {}");
		MockMaker aMockMaker = new MockMaker(classStructure, myMockCodeWriter, new TestConfiguration());
    aMockMaker.setIncludePackage(true);
		aMockMaker.make();

		verify();
	}

	public void testMakeMockFromSourceWithCorrectNamedParameters() throws Exception {
		myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationCounter","myEatThingsCalls","new ExpectationCounter(\"XMockInterfaceWithMethods.eatThings(String, int)\")");
    myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ReturnValues","myActualEatThingsReturnValues","new VoidReturnValues(\"XMockInterfaceWithMethods.eatThings(String, int)\", true)");
		myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myEatThingsParameter0Values","new ExpectationList(\"XMockInterfaceWithMethods.eatThings(String, int) : java.lang.String name\")");
		myMockCodeWriter.addExpectedWriteInstanceVariableDeclarationValues("ExpectationList","myEatThingsParameter1Values","new ExpectationList(\"XMockInterfaceWithMethods.eatThings(String, int) : int number\")");

		ClassStructure classStructure = createSourceClassStructure(
			"package mockmaker.tests; public interface InterfaceWithMethods {"
			+ "void eatThings(String name, int number);"
			+ "}");

		MockMaker aMockMaker = new MockMaker(classStructure, myMockCodeWriter, new TestConfiguration());
		aMockMaker.make();

		verify();
	}

	private ClassStructure createSourceClassStructure(String in) {
		JavaDocBuilder builder = new JavaDocBuilder();
		builder.addSource(new StringReader(in));
		JavaClass cls = builder.getSources()[0].getClasses()[0];
		ClassStructure classStructure = new SourceClassStructure(cls, null);
		return classStructure;
	}


}