package mockmaker.tests;

import junit.framework.TestCase;
import mockmaker.ReturnValues;
import mockmaker.VoidReturnValues;
import mockmaker.ExceptionalReturnValue;

public class VoidReturnValuesTest extends TestCase {

	private ReturnValues r;

	private class MyException extends Exception {
	}

	private class OtherException extends Exception {
	}

	public VoidReturnValuesTest(String s) {
		super(s);
	}

	protected void setUp() throws Exception {
		super.setUp();
	}

	public void testDefaultBehavior() throws Exception {
		r = new VoidReturnValues(false);
		assertNull(r.getNext());
		assertNull(r.getNext());
		assertNull(r.getNext());
		assertNull(r.getNext());
		assertNull(r.getNext());
		assertNull(r.getNext());
	}

	public void testAddOneException() throws Exception {
		r = new VoidReturnValues(false);
		Exception e = new MyException();
		r.add(new ExceptionalReturnValue(e));

		ExceptionalReturnValue next = (ExceptionalReturnValue)r.getNext();
		assertEquals(e, next.getException());

		assertNull(r.getNext());
		assertNull(r.getNext());
	}

	public void testAddOneRepetitiveException() throws Exception {
		r = new VoidReturnValues(true);
		Exception e = new MyException();
		r.add(new ExceptionalReturnValue(e));

		assertEquals(e, ((ExceptionalReturnValue)r.getNext()).getException());
		assertEquals(e, ((ExceptionalReturnValue)r.getNext()).getException());
		assertEquals(e, ((ExceptionalReturnValue)r.getNext()).getException());
		assertEquals(e, ((ExceptionalReturnValue)r.getNext()).getException());
	}

	public void testAddExceptionAndNoExceptionMixture() throws Exception {
		r = new VoidReturnValues(false);
		Exception e1 = new MyException();
		Exception e2 = new MyException();
		Exception e3 = new OtherException();

		r.add(null);
		r.add(new ExceptionalReturnValue(e1));
		r.add(new ExceptionalReturnValue(e1));
		r.add(null);
		r.add(new ExceptionalReturnValue(e2));
		r.add(new ExceptionalReturnValue(e3));
		r.add(null);
		r.add(null);
		r.add(new ExceptionalReturnValue(e3));


		assertNull(r.getNext());
		assertEquals(e1, ((ExceptionalReturnValue)r.getNext()).getException());
		assertEquals(e1, ((ExceptionalReturnValue)r.getNext()).getException());
		assertNull(r.getNext());
		assertEquals(e2, ((ExceptionalReturnValue)r.getNext()).getException());
		assertEquals(e3, ((ExceptionalReturnValue)r.getNext()).getException());
		assertNull(r.getNext());
		assertNull(r.getNext());
		assertEquals(e3, ((ExceptionalReturnValue)r.getNext()).getException());

		assertNull(r.getNext());
		assertNull(r.getNext());
	}

}
