package org.subconscious.mockmaker.gui;


import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;

import mockmaker.*;



/**
 * Title:        GUIFrame
 * Description:  The gui frame for MockMaker.
 * @author       <a href="mailto:scott@vlaminck.com">Scott Vlaminck</a>
 */

public class GUIFrame extends JFrame {


  // do we allow only interfaces?
  // if future versions of MockMaker, this could be set using
  // a radio button or checkbox
  private boolean allowInterfacesOnly = true;

  private File rootDirectory = null;
  private File outputFile = null;
  private File inputFile = null;
  private DumbClassLoader loader = null;
  private AlertDialog dialog = null;



  JPanel contentPane;
  JButton jButton_make = new JButton();
  JLabel jLabel_existingClassFile = new JLabel();
  JTextField jTextField_filepath = new JTextField();
  JButton jButton_browseForNewClass = new JButton();
  JLabel jLabel_newClass = new JLabel();
  JFileChooser inputFileChooser = null;
  JFileChooser outputFileChooser = null;
  JTextField jTextField_existingClassFile = new JTextField();
  JButton jButton_browseForExistingClassFile = new JButton();
  JTextField jTextField_existingClassName = new JTextField();
  JLabel jLabel_existingClassName = new JLabel();
  GridBagLayout gridBagLayout1 = new GridBagLayout();

  /**Construct the frame*/
  public GUIFrame() {
	this( null );
  }  

  public GUIFrame( File rootFileChooserDirectory ) {
	enableEvents(AWTEvent.WINDOW_EVENT_MASK);
	try {
	  jbInit( rootFileChooserDirectory );
	}
	catch(Exception e) {
	  e.printStackTrace();
	}
  }  



  /**Component initialization*/
  private void jbInit( File rootFileChooserDirectory ) throws Exception  {
	setRootDirectory( rootFileChooserDirectory );
	jbInit();
  }  


  /**Component initialization*/
  private void jbInit() throws Exception  {

	loader = new DumbClassLoader( this.getClass().getClassLoader() );

	java.net.URL image = GUIFrame.class.getResource("/mockmaker.gif");
	if( image != null ) {
	  setIconImage(Toolkit.getDefaultToolkit().createImage( image ));
	}

	setupFileChoosers();

	contentPane = (JPanel) this.getContentPane();
	jButton_make.setToolTipText("Make Mock Object");
	jButton_make.setText("Make");
	jButton_make.addActionListener(new java.awt.event.ActionListener() {
	  public void actionPerformed(ActionEvent e) {
		jButton_make_actionPerformed(e);
	  }
	});
	contentPane.setLayout(gridBagLayout1);
	this.setSize(new Dimension(379, 292));
	this.setTitle(" MockMaker");
	jLabel_existingClassFile.setText("Create MockObject for:");
	jButton_browseForNewClass.setToolTipText("");
	jButton_browseForNewClass.setText("Browse");
	jButton_browseForNewClass.addActionListener(new java.awt.event.ActionListener() {
	  public void actionPerformed(ActionEvent e) {
		jButton_browseForNewClass_actionPerformed(e);
	  }
	});
	contentPane.setMinimumSize(new Dimension(270, 200));
	contentPane.setPreferredSize(new Dimension(270, 200));
	jLabel_newClass.setText("Create new class:");
	jButton_browseForExistingClassFile.addActionListener(new java.awt.event.ActionListener() {
	  public void actionPerformed(ActionEvent e) {
		jButton_browseForExistingClassFile_actionPerformed(e);
	  }
	});
	jButton_browseForExistingClassFile.setText("Browse");
	jTextField_filepath.addActionListener(new java.awt.event.ActionListener() {
	  public void actionPerformed(ActionEvent e) {
		jTextField_filepath_actionPerformed(e);
	  }
	});
	jTextField_existingClassFile.addActionListener(new java.awt.event.ActionListener() {
	  public void actionPerformed(ActionEvent e) {
		jTextField_existingClassFile_actionPerformed(e);
	  }
	});
	jTextField_existingClassName.setEditable(false);
	jLabel_existingClassName.setText("Classname (based on file chosen above):");
	jTextField_filepath.setToolTipText("");
	contentPane.add(jTextField_existingClassFile,  new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 51, 0, 0), 170, 3));
	contentPane.add(jButton_browseForExistingClassFile,  new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
			,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 7, 0, 52), 18, -3));
	contentPane.add(jLabel_existingClassFile,  new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(39, 51, 0, 10), 41, 1));
	contentPane.add(jLabel_existingClassName,  new GridBagConstraints(0, 2, 2, 1, 0.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(16, 51, 0, 77), 21, 1));
	contentPane.add(jTextField_filepath,  new GridBagConstraints(0, 5, 1, 1, 1.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 51, 0, 0), 170, 3));
	contentPane.add(jLabel_newClass,  new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(38, 51, 0, 10), 65, 1));
	contentPane.add(jButton_make,  new GridBagConstraints(0, 6, 1, 1, 0.0, 0.0
			,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(20, 51, 27, 99), 12, 3));
	contentPane.add(jButton_browseForNewClass,  new GridBagConstraints(1, 5, 1, 1, 0.0, 0.0
			,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 7, 0, 52), 18, -3));
	contentPane.add(jTextField_existingClassName,  new GridBagConstraints(0, 3, 2, 1, 1.0, 0.0
			,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 51, 0, 52), 272, -1));
  }  

  /**
   * Initialize our FileChooser classes for the two 'Browse' buttons.
   */
  private void setupFileChoosers() {

	File rootDirectory = getRootDirectory();
	if( rootDirectory == null ) {
	  inputFileChooser = new JFileChooser();
	  outputFileChooser = new JFileChooser();
	} else {
	  inputFileChooser = new JFileChooser( rootDirectory );
	  outputFileChooser = new JFileChooser( rootDirectory );
	}


	inputFileChooser.addChoosableFileFilter(
		new ExtensionFileFilter( ".class", "Java class files (*.class)" )
	);

	outputFileChooser.addChoosableFileFilter(
		new ExtensionFileFilter( ".java", "Java source files (*.java)" )
	);


//
//  Pulled this out into an inner class.
//
//    inputFileChooser.addChoosableFileFilter(
//        new javax.swing.filechooser.FileFilter() {
//          public boolean accept(File f) {
//            return f != null && ( f.isDirectory() || f.getName().endsWith( ".class" ) );
//          }
//          public String getDescription() { return "Java class files (*.class)"; }
//        }
//    );
//

	return;
  }  


  /**Overridden so we can exit when window is closed*/
  protected void processWindowEvent(WindowEvent e) {
	super.processWindowEvent(e);
	if (e.getID() == WindowEvent.WINDOW_CLOSING) {
	  System.exit(0);
	}
  }  

  /**
   * action performed for 'Make' Button
   */
  void jButton_make_actionPerformed(ActionEvent e) {
	make();
  }  


  /**
   * Generate the code.
   */
  private void make() {

	String input = jTextField_existingClassName.getText();
	String output = jTextField_filepath.getText();

	if( input == null || "".equals( input ) ) {
	  alert( "You must select an interface." );
	  return;
	}

	if( output == null || "".equals( output ) ) {
	  alert( "You must select an output source file." );
	  return;
	}

	// make
	try {
	  // input/output files are good, make our MockObject
	  MockMaker mm = new MockMaker( input,
									new RealCodeWriter( getPrintWriter( outputFile ) ),
									new RealConfiguration(RealConfiguration.DEFAULT_CONFIGURATION_FILENAME),
									loader
								  );
	  mm.make();


	  alert( "Done" );

	} catch( ClassNotFoundException cnf ) {
	  alert( "MockMaker Error. ClassNotFoundException: " + cnf.getMessage() );
	} catch( java.io.FileNotFoundException fnf ) {
	  alert( "Output File Not Found: " + fnf.getMessage() );
	} catch( java.io.IOException io ) {
	  alert( "Output File Exception: " + io.getMessage() );
	}

	return;
  }    


  /**
   * Utility method to create a PrintWriter given a File.
   */
  private PrintWriter getPrintWriter( File f ) throws java.io.IOException, java.io.FileNotFoundException {

	if( !f.exists() ) {
	  boolean mkdirs = f.getParentFile().mkdirs();
	  boolean createFile = f.createNewFile();
	}

	FileOutputStream fos = new FileOutputStream( f );
	BufferedOutputStream buf = new BufferedOutputStream( fos );

	return new PrintWriter( buf );

  }  

  void jButton_browseForNewClass_actionPerformed(ActionEvent e) {

	if (JFileChooser.APPROVE_OPTION == outputFileChooser.showOpenDialog(this)) {
	  outputFile = outputFileChooser.getSelectedFile();

	  jTextField_filepath.setText( outputFile.getAbsolutePath() );
	}

	return;
  }  


  void jButton_browseForExistingClassFile_actionPerformed(ActionEvent e) {
	if (JFileChooser.APPROVE_OPTION == inputFileChooser.showOpenDialog(this)) {
	  inputFile = inputFileChooser.getSelectedFile();

	  jTextField_existingClassFile.setText( inputFile.getAbsolutePath() );
	  jTextField_existingClassName.setText( determineFullClassName( inputFile ) );
	}

	return;
  }  



  void jTextField_filepath_actionPerformed(ActionEvent e) {

	String newFileText = jTextField_filepath.getText();
	if( outputFile == null || (!newFileText.equals( outputFile.getAbsolutePath() )) ) {

	  if( newFileText != null && !"".equals( newFileText ) ) {
		outputFile = new File( newFileText );
	  }
	}
	return;
  }  

  void jTextField_existingClassFile_actionPerformed(ActionEvent e) {

	String classFileText = jTextField_existingClassFile.getText();
	if( inputFile == null || (!classFileText.equals( inputFile.getAbsolutePath() )) ) {

	  if( classFileText != null && !"".equals( classFileText ) ) {
		inputFile = new File( classFileText );
	  }
	}
	return;
  }  

  /**
   * @param File class file
   * @return String class name
   */
  private String determineFullClassName( File classFile ) {

	Class c = null;

	try {
	  c = loader.defineAClass( classFile );
	} catch( Exception invalidFileException ) {
	  alert( "Invalid File: " + invalidFileException.getMessage() );
	  return "";
	} catch( LinkageError link ) {
	  // already selected and loaded this class, just get it from memory
	  // message == "duplicate class definition: mockmaker/CodeWriter"
	  int colon = link.getMessage().indexOf( ":" );

	  // take account of colon and the space, and switch to dot notation
	  //  the slash is always forward (it is not platform dependent)
	  String className = link.getMessage().substring( colon + 2 ).replace( '/', '.' );

	  // now load the class from memory
	  try {
		c = Class.forName( className, true, loader );
	  } catch( Exception e ) {}
	}

	// if something went wrong in there, just quit now
	if( c == null ) return "";


	if( allowInterfacesOnly && !c.isInterface() ) {
	  alert( "You must select an Interface. Classes are not allowed." );
	  return "";
	}

	return c.getName();
  }  


  /**
   * Generate an alert box.
   */
  private void alert( String message ) {

	AlertDialog alert = getDialog();
	alert.setMessage( message );
	alert.show();

  }  



  private AlertDialog getDialog() {
	if( dialog == null ) {
	  dialog = new AlertDialog( this, "Alert", true );

	  //Center the window
	  Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	  Dimension dialogSize = dialog.getSize();
	  if (dialogSize.height > screenSize.height) {
		dialogSize.height = screenSize.height;
	  }
	  if (dialogSize.width > screenSize.width) {
		dialogSize.width = screenSize.width;
	  }
	  dialog.setLocation((screenSize.width - dialogSize.width) / 2, (screenSize.height - dialogSize.height) / 2);

	}
	return dialog;
  }  

  File getRootDirectory() {
//    if( rootDirectory == null ) {
//      alert( "rootDirectory == null" );
//    } else {
//      alert( "rootDirectory == " + rootDirectory.getAbsolutePath() );
//    }
	return rootDirectory;
  }  

  void setRootDirectory( File fileChooserRootDirectory ) {
	rootDirectory = fileChooserRootDirectory;
  }  



  /**
   * Our extension File Filter.
   */
   class ExtensionFileFilter extends javax.swing.filechooser.FileFilter {

	  String extension = null;
	  String description = null;

	  public ExtensionFileFilter() {}
	  public ExtensionFileFilter( String fileExtension, String filterDescription ) {
		setExtension( fileExtension );
		setDescription( filterDescription );
	  }

	  public void setExtension( String fileExtension ) {
		extension = fileExtension;
	  }

	  public String getExtension() {
		return extension;
	  }

	  public void setDescription( String filterDescription ) {
		description = filterDescription;
	  }

	  public String getDescription() {
		if( description == null ) description = "";
		return description;
	  }

	  public boolean accept(File f) {
		if( extension == null ) return true;
		return f != null && ( f.isDirectory() || f.getName().endsWith( extension ) );
	  }
	}





  /**
   * Our special classloader.
   */
  private class DumbClassLoader extends ClassLoader {

	/**
	 * Constructor.
	 * @param ClassLoader
	 */
	public DumbClassLoader( ClassLoader loader ) {
	  super( loader );
	}

	/**
	 * Create a public interface to define the class.
	 * @param String The name of the class we hope to define. <code>null</code> if unknown.
	 * @param byte[] The compiled bytes of the class to define.
	 * @param int Offset of where to start in the byte array.
	 * @param int Length of the byte array to use in definition of the class.
	 * @return Class The class that was defined.
	 */
	public Class defineAClass( String name, byte[] b, int off, int len ) {
	  return super.defineClass( name, b, off, len );
	}

	/**
	 * Create a public interface to define the class.
	 * @param File The file of the class we hope to define.
	 * @return Class The class that was defined.
	 */
	public Class defineAClass( File classFile ) throws InvalidFileException {

	  // check validity of file
	  if( classFile == null ) throw new InvalidFileException( "Null file." );
	  if( !classFile.getName().endsWith( ".class" ) ) throw new InvalidFileException( "Not a java class file." );


	  String name = getClassName( classFile );
	  name = null;

	  // the actual bytes of our java file
	  byte[] classBytes;

	  try {
		// the file, so we can get the compiled bytes
		FileInputStream in = new FileInputStream( classFile );

		// get the bytes, create an array, then read 'em in
		classBytes = new byte[in.available()];
		in.read(classBytes);

	  } catch( java.io.FileNotFoundException fnf ) {
		throw new InvalidFileException( fnf.getMessage() );
	  } catch( java.io.IOException io ) {
		throw new InvalidFileException( io.getMessage() );
	  }

	  return super.defineClass( name, classBytes, 0, classBytes.length );
	}

	private String getClassName( File classFile ) {


	  // determine package (if any)
	  String path = classFile.getParent();

	  String packageName = null;

	  int packageBegin = path.indexOf( "classes" );
	  if( packageBegin > -1 ) {
		packageName = path.substring( packageBegin + 8 )
						  .replace( File.separator.charAt(0), '.' );
	  }

	  // retrieve the Java class name
	  int dot = classFile.getName().indexOf( "." );
	  String name = classFile.getName().substring( 0, dot );

	  if( packageName != null ) {
		name = packageName + "." + name;
	  }

	  return name;
	}


	/**
	 * Our Loader's specific Exception
	 */
	protected class InvalidFileException extends Exception {
	  public InvalidFileException() {
		super();
	  }
	  public InvalidFileException( String message ) {
		super( message );
	  }
	}

  }

}